/* Author: Tobi Vollebregt */

/*  Trolls'n Towers: Tobi Vollebregt's TINS Is Not SpeedHack 2005 entry.
 *  Copyright (C) 2005  Tobi Vollebregt
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See `License.txt', which contains a verbatim copy of the
 *  GNU General Public License, for details.
 *
 *  Please send your reaction to: tobivollebregt@hotmail.com
 */

// more like tools or common, not only debug

#ifndef __tjv_debug_h__
#define __tjv_debug_h__

#include <exception>

class Error: public std::exception
{
	char msg[512];
 public:
	Error(const char* fmt, ...) throw();
	const char* what() const throw() { return msg; }
};

#ifdef DEBUGMODE
 #undef ASSERT
 #define ASSERT(x) if(!(x)) throw Error("Assertion failed at line %d of file %s:\n%s", __LINE__, __FILE__, #x);
#endif

// Use this to wipe out stl containers containing pointers to single objects.
template <class T>
inline void STLWipe (T begin, const T &end)
{
	for(; begin != end; ++begin) delete (*begin);
}

// Use this to wipe out stl containers containing pointers to arrays.
template <class T>
inline void STLWipeArray (T begin, const T&end)
{
	for(; begin != end; ++begin) delete[] (*begin);
}

#define RANDOM(min, max) ((((max) - (min)) * rand() / RAND_MAX) + (min))

#ifndef PI
 #define PI 3.1415926535897932384626433832795
#endif // !defined (PI)

#define SIZEOF(a) ((int) (sizeof(a) / sizeof(a[0])))

#endif // !defined(__tjv_debug_h__)
