/* Author: Tobi Vollebregt */

/*  Trolls'n Towers: Tobi Vollebregt's TINS Is Not SpeedHack 2005 entry.
 *  Copyright (C) 2005  Tobi Vollebregt
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See `License.txt', which contains a verbatim copy of the
 *  GNU General Public License, for details.
 *
 *  Please send your reaction to: tobivollebregt@hotmail.com
 */

#include <allegro.h>
#include <math.h>
#include "data.h"
#include "enemy.h"
#include "player.h"
#include "particle.h"
#include "troll.h"

#define GRAVITY (-0.1)
#define XACCEL  (0.1)
#define YACCEL  (0.2)
#define XMULT   (0.99)
#define YMULT   (0.95)
#define XSPEED  (-5.0)

#define PLOCX 0 //77        // Propellor
#define PLOCY 19
#define PSIZE 8
#define SLOCX 76 //1         // Rook (TODO)
#define SLOCY 19

#define STATE_ATTACK 0
#define STATE_RETREAT 1
#define STATE_DEAD 2

Enemy::Enemy()
{
	x = the_player->get_x() + double_buffer->w + rand() % double_buffer->w;
	y = MAX(100, int(the_player->get_y()) - double_buffer->h) + rand() % (2*double_buffer->h);
	xs = RANDOM(-1.0, -10.0);
	ys = 0.0;
	state = 0;
}

bool Enemy::update()
{
	BITMAP* b = (BITMAP*) data[BMP_OLDPLANE].dat;
	//float angle = atan2(dy, dx);
	//float roll = atan2(ys, xs); // our angle

	GameObject* obj;
	for (int i = 0; i < GetNumGameObjects(); ++i)
	{
		if ((obj = GetGameObject(i)) && obj != this)
		{
			if (obj->hit(int(x), int(y)))
			{
				if (dynamic_cast<Troll*>(obj))
				{
					xs = -0.5 * xs;
					x += 3; // otherwise instability could result.
				}
				if (state != STATE_DEAD) the_player->inckill(false);
				state = STATE_DEAD;
			}
			else if (state != STATE_DEAD)
			{
				Enemy* en = dynamic_cast<Enemy*>(obj);
				if (en)
				{
					if (en->x < x && en->x > x - 150 && fabs(en->y - y) < b->h)
					{
						if (y > en->y) ys += YACCEL;
						else ys -= YACCEL;
						xs += XACCEL;
					}
				}
			}
		}
	}

	x += xs;
	y += ys;

	if (state != STATE_DEAD)
	{
		AddGameObject2(new Particle(x+SLOCX-b->w/2,y-SLOCY+b->h/2,RANDOM(0.0,1.0),RANDOM(-1.0,1.0),10,7,
			makecol(128, 96, 64)));
		xs -= XACCEL;
		xs *= XMULT;
		ys *= YMULT;
	}

	   else {
			AddGameObject2(new Particle(x-b->w/2+rand()%b->w,y-b->h/2+rand()%b->h,RANDOM(-1.0,1.0),RANDOM(-1.0,1.0),40,10,
				explosion_color[rand()&127]));
			ys += GRAVITY;
		}

	rotor += 23;    // auto-wrap-around

	return int(x) > the_player->get_x() - double_buffer->w && int(y) > -b->h/2;
}

bool Enemy::hit(int hx, int hy)
{
	BITMAP* b = (BITMAP*) data[BMP_OLDPLANE].dat;

	int bx = hx - int(x) + b->w/2;
	int by = b->h/2 + int(y) - hy;
	int c = getpixel(b, bx, by);

	if (c >= 0 && c != bitmap_mask_color(b))
	//if (hx > int(x) - b->w/2 && hx < int(x) + b->w/2 &&
	//    hy > int(y) - b->h/2 && hy < int(y) + b->h/2)
	{
		if (state != STATE_DEAD)
		{
			for (int i = 0; i < 25; ++i)
			{
				AddGameObject2(new Particle(x-b->w/2+rand()%b->w,y-b->h/2+rand()%b->h,RANDOM(-5.0,5.0),RANDOM(-5.0,5.0),100,10,
					explosion_color[rand()&255]));
			}
			state = STATE_DEAD;
			the_player->inckill();
		}
		return true;
	}
	return false;
}

void Enemy::draw(int xofs, int yofs)
{
	BITMAP* b = (BITMAP*) data[BMP_OLDPLANE].dat;

	// and another sine wave !!!

	draw_sprite(double_buffer, b, int(x) - b->w/2 - xofs,
		double_buffer->h - int(y) - b->h/2 + yofs);

	// teken propellor
	vline(double_buffer, int(x)+PLOCX-b->w/2-xofs, double_buffer->h-int(y)-b->h/2+PLOCY+int(sin(PI*rotor/128)*PSIZE)+yofs,
		double_buffer->h-int(y)-b->h/2+PLOCY-int(sin(PI*rotor/128)*PSIZE)+yofs, makecol(255, 255, 255));
}
