/* Author: Tobi Vollebregt */

/*  Trolls'n Towers: Tobi Vollebregt's TINS Is Not SpeedHack 2005 entry.
 *  Copyright (C) 2005  Tobi Vollebregt
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See `License.txt', which contains a verbatim copy of the
 *  GNU General Public License, for details.
 *
 *  Please send your reaction to: tobivollebregt@hotmail.com
 */

/*  Trolls'n Towers: Tobi Vollebregt's TINS Is Not SpeedHack 2005 entry.
 *  Copyright (C) 2005  Tobi Vollebregt
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See `License.txt', which contains a verbatim copy of the
 *  GNU General Public License, for details.
 *
 *  Please send your reaction to: tobivollebregt@hotmail.com
 */

#include <allegro.h>
#include <time.h>
#include <vector>
#include "data.h"
#include "debug.h"
#include "object.h"
#include "skybox.h"
#include "map.h"
#include "player.h"
#include "enemy.h"
#include "troll.h"
#include "hiscore.h"
//#include "particle.h"

	// timer
static volatile int speed_counter;
static void inc_speed_counter() { ++speed_counter; }
END_OF_STATIC_FUNCTION(inc_speed_counter)

	// fps proc
static volatile int fps, frame;
static void fps_proc() { fps = frame; frame = 0; }
END_OF_STATIC_FUNCTION(fps_proc);

	// close button hook instead of this silly forced shutdown msg
static volatile bool close_hook_done;
static void close_hook() { close_hook_done = true; }
END_OF_STATIC_FUNCTION(close_hook);

BITMAP* double_buffer;    // double buffer / back buffer
DATAFILE* data;     // global datafile / contains every data thing
int explosion_color[256];
bool play_again;


static void initialize()
{
	LOCK_VARIABLE(speed_counter);
	LOCK_FUNCTION(inc_speed_counter);

	LOCK_VARIABLE(frame);
	LOCK_VARIABLE(fps);
	LOCK_FUNCTION(fps_proc);

	LOCK_VARIABLE(close_hook_done);
	LOCK_FUNCTION(close_hook);

	if (allegro_init() < 0)
		throw Error("Couldn't initialize Allegro.");

	set_window_title("Troll'n Tower");
	set_window_close_hook(close_hook);

	int color_depth = desktop_color_depth();

	if (color_depth == 8)
		throw Error("Desktop must have more than 256 colors.");

	if (!color_depth) color_depth = 16;
	set_color_depth(color_depth);

	if (set_gfx_mode(GFX_AUTODETECT_WINDOWED, 640, 480, 0, 0) < 0)
	{
		// if color_depth is 15 or 16 directx does return 16 always and
		// the other mode fails to initialize, so we try both in this case.
		if (color_depth != 16 ||        // ha anti n00b code ;-)
			(set_color_depth((color_depth = 15)),
			set_gfx_mode(GFX_AUTODETECT_WINDOWED, 640, 480, 0, 0) < 0))
			throw Error("Error setting %dx%dx%d graphics mode:\n%s",
						640, 480, color_depth, allegro_error);
	}

	if (!(double_buffer = create_bitmap(SCREEN_W, SCREEN_H)))
		throw Error("Not enough memory for double buffering.");

	if (install_mouse() < 0)
		throw Error("Couldn't install mouse.");
	if (install_timer() < 0)
		throw Error("Couldn't install timer.");
	if (install_keyboard() < 0)
		throw Error("Couldn't install keyboard.");

	{   // load datafile
	// doesn't work with XP
		/*char tmp1[512], tmp2[512];
		get_executable_name(tmp1, sizeof(tmp1));
		replace_filename(tmp2, tmp1, "data.dat", sizeof(tmp2));
		fix_filename_path(tmp1, tmp2, sizeof(tmp1));*/

		if (!(data = load_datafile("data.dat")))
			throw Error("Couldn't load datafile \"data.dat\":\n%s", strerror(errno));
	}

	for(int i=0; i<64; i++) {
		explosion_color[i] = makecol(i<<2, 0, 0);
	}

	for(int i=64; i<128; i++) {
		explosion_color[i] = makecol(255,(i-64)<<2,0);
	}

	for(int i=128; i<256; i++) {
		explosion_color[i] = makecol(255, 255, (i-128)<<1);
	}
	//InitParticleSystem(0);
}

std::vector<GameObject*> game_objects;
std::vector<GameObject*> game_objects2; // no coll. det. (ie particles)
std::vector<GameObject*> to_be_added;
std::vector<GameObject*> to_be_added2; // no coll. det. (ie particles)

Player* the_player;
Map* the_map;
CHiscore* the_hiscore;

void AddGameObject(GameObject* g)
{
	to_be_added.push_back(g);
}

void AddGameObject2(GameObject* g)
{
	to_be_added2.push_back(g);
}

// this wraps std::vector, so that I don't need to include it's 100k lines in each source file
int GetNumGameObjects() { return game_objects.size(); }
GameObject* GetGameObject(int i) { return game_objects[i]; }

int trollenteller()
{
	BITMAP* b = (BITMAP*) data[BMP_TROLL].dat;

	float max_height =0.0;
	GameObject* obj;
	Troll* tr;

	for (int i = 0; i < GetNumGameObjects(); ++i)
		if ((obj = GetGameObject(i)) && (tr = dynamic_cast<Troll*>(obj)))
			if (tr->get_y()+b->h > max_height)
				max_height = tr->get_y()+b->h;

	return int(max_height / b->h * 60);
}

void do_game()
{
	std::vector<GameObject*>::iterator it;
	int x = 0, y = 0, tt[50];
	int tt_pos = 0;

	AddGameObject(new Skybox);
	AddGameObject((the_map = new Map));
	AddGameObject((the_player = new Player));
	the_hiscore = NULL;

	install_int_ex(inc_speed_counter, BPS_TO_TIMER(50));
	install_int_ex(fps_proc, BPS_TO_TIMER(1));

	while (!close_hook_done && !key[KEY_ESC] && !play_again)
	{
		++frame;

		while (speed_counter > 0 && !close_hook_done && !key[KEY_ESC] && !play_again)
		{
			// update game logic

			// trollenteller() must be the same 1 second long
			if (++tt_pos == SIZEOF(tt)) tt_pos = 0;
			tt[tt_pos] = trollenteller();

			if ((rand() & 7) == 0) game_objects.push_back(new Enemy);

			for (it = game_objects.begin(); it != game_objects.end(); ++it)
				if ((**it).update()) to_be_added.push_back(*it);
				else { delete *it; *it = NULL; }
			for (it = game_objects2.begin(); it != game_objects2.end(); ++it)
				if ((**it).update()) to_be_added2.push_back(*it);
				else { delete *it; *it = NULL; }

			game_objects.clear();
			game_objects2.clear();
			game_objects.swap(to_be_added);
			game_objects2.swap(to_be_added2);

			bool ok =true;
			for (int i =1 ; i < SIZEOF(tt); ++i)
				if ((tt[i]&~3)!=(tt[i-1]&~3)) { ok = false; break; }

			if (the_player->is_dead() && !the_hiscore && ok)
			{
				game_objects2.push_back((the_hiscore = new CHiscore));
				the_hiscore->InitDialog(tt[0]);
			}

			--speed_counter;
		}

		// redraw

		x = the_player->get_viewport_x();
		y = the_player->get_viewport_y();
		x = MID(0, x, the_map->get_width() - double_buffer->w);
		y = MAX(0, y);

		for (it = game_objects.begin(); it != game_objects.end(); ++it)
			(**it).draw(x, y);
		for (it = game_objects2.begin(); it != game_objects2.end(); ++it)
			(**it).draw(x, y);

		//DrawParticleSystem();

		if (fps)
		{
			text_mode(-1);
			//textprintf(double_buffer, font, 0, 0, makecol(255, 255, 255), "%d fps, %d collidable objects, %d uncollidable objects", fps, game_objects.size(), game_objects2.size());
			textprintf(double_buffer, font, 0, 0, makecol(255, 255, 255), "%d fps", fps);
		}
		text_mode(-1);
		textprintf(double_buffer, font, 10, 40, makecol(255, 255, 255), "troll tower : %d feet", tt[tt_pos]);
		textprintf(double_buffer, font, 10, 50, makecol(255, 255, 255), "      frags : %d", the_player->getkills());

		blit(double_buffer, screen, 0, 0, 0, 0, double_buffer->w, double_buffer->h);

		// improved (read: reduced) CPU usage
		while (!speed_counter) yield_timeslice();
	}

	remove_int(inc_speed_counter);
	remove_int(fps_proc);

	STLWipe(game_objects.begin(), game_objects.end());
	STLWipe(game_objects2.begin(), game_objects2.end());
	game_objects.clear();
	game_objects2.clear();
	// to_be_added? is already empty here
}

int main()
{
	srand((unsigned) time(0));

	try
	{
		initialize();

		do
		{
			play_again = false;
			do_game();
		}
		while (play_again);
	}
	catch (const std::exception& e)
	{
		if (screen) set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
		set_window_title("FATAL ERROR");
		allegro_message("%s", e.what());
		return 1;
	}

	return 0;
}
END_OF_MAIN()
