/* Author: Tobi Vollebregt */

/*  Trolls'n Towers: Tobi Vollebregt's TINS Is Not SpeedHack 2005 entry.
 *  Copyright (C) 2005  Tobi Vollebregt
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See `License.txt', which contains a verbatim copy of the
 *  GNU General Public License, for details.
 *
 *  Please send your reaction to: tobivollebregt@hotmail.com
 */

#include <allegro.h>
#include <math.h>
#include "data.h"
#include "map.h"
#include "player.h"
#include "rocket.h"
#include "particle.h"
#include "troll.h"

#define GRAVITY (-0.2)
#define XACCEL  (0.5)
#define YACCEL  (0.5)
#define XMULT   (0.95)
#define YMULT   (0.95)
#define GUNX 58
#define GUNY 22

Player::Player()
{
	x = 50000;
	y = 50;
	xs =0;
	ys = 0;
	dead = false;
	kills = 0;
	num_trolls = 0;
	troll_unlocked = false;
	init_kills =0;
	kill_timer = 0;
	zoom = 0;
	tkills=0;
	num_monsterkill = 0;
}

bool Player::hit(int hx, int hy)
{
	BITMAP* b = (BITMAP*) data[BMP_HELICOPTER].dat;

	int bx = hx - int(x) + b->w/2;
	int by = b->h/2 + int(y) - hy;
	int c = getpixel(b, bx, by);

	if (c >= 0 && c != bitmap_mask_color(b))
	//if (hx > int(x) - b->w/2 && hx < int(x) + b->w/2 &&
	//    hy > int(y) - b->h/2 && hy < int(y) + b->h/2)
	{
		return dead = true;
	}
	return false;
}

bool Player::update()
{
	BITMAP* b = (BITMAP*) data[BMP_HELICOPTER].dat;

	GameObject* obj;
	for (int i = 0; i < GetNumGameObjects(); ++i)
	{
		if ((obj = GetGameObject(i)) && obj != this)
		{
			if (obj->hit(int(x), int(y)))
			{
				if (dynamic_cast<Troll*>(obj))
				{
					xs = -0.5 * xs;
					x += 3*xs; // otherwise instability could result.
				}
				dead = true;
			}
		}
	}

	x += xs;
	y += ys;

	ys += GRAVITY;

	if (kill_timer)
	{
		--kill_timer;
		if (kills - init_kills >= (4+4*num_monsterkill)*(1<<(num_monsterkill/3)))
		{
			++num_monsterkill;
			zoom = 1;
		}
	}
	else num_monsterkill = 0;

	if (zoom) {
		zoom += 0.2;
		if (zoom > 6) zoom = 0;
	}

	if (dead)
	{
		AddGameObject2(new Particle(x-b->w/2+rand()%b->w,y-b->h/2+rand()%b->h,RANDOM(-2.5,2.5),RANDOM(-2.5,2.5),100,10,
			explosion_color[rand()&127]));
	}
	else
	{
		if (key[KEY_LEFT]) xs -= XACCEL;
		if (key[KEY_RIGHT]) xs += XACCEL;
		if (key[KEY_UP]) ys += YACCEL;
		if (key[KEY_DOWN]) ys -= YACCEL;
		if (key[KEY_SPACE])
		{
			if (!shoot_pressed)
			{
				AddGameObject2(new Rocket(x+GUNX-b->w/2, y-GUNY+b->h/2, xs, ys));
				shoot_pressed = true;
			}
		}
		else shoot_pressed = false;

		if (tkills == 10) { troll_unlocked = true; tkills=0;}

		if (key[KEY_Z] && troll_unlocked)
		{
			troll_unlocked = false;
			AddGameObject(new Troll(x, y+double_buffer->h));
		}
	}

	if (dead)
	{
		xs *= 0.99;
		ys *= 0.99;
	}
	else
	{
		xs *= XMULT;
		ys *= YMULT;
	}

	// these automagically wrap around
	rotor += 11;
	srotor += 23;

	if (y < ((BITMAP*) data[BMP_HELICOPTER].dat)->h/2)
	{
		ys = -0.5 * ys;
		y = ((BITMAP*) data[BMP_HELICOPTER].dat)->h/2;
	}

	/*if (hit_ground((BITMAP*) data[BMP_HELICOPTER].dat, int(x), int(y)))
	{
		KaBoom(int(x), int(y), 20, 2);
		x = old_x;
		y = old_y + 1.0;
		ys *= -1.0;
		//xs *= -1.0;
	}*/

	return true;
}

/*Zelfs nog een stukje paraclone
==================================
	if(h->xs>0)
	{
		draw_sprite(page[pn], h->bmp, h->x,h->y);
		if(h->type == 0) {
			// teken staartrotor
			line(page[pn], cos(pi*h->srotor/180)*SRSIZE+h->x+SRLOCX,sin(pi*h->srotor/180)*SRSIZE+h->y+SRLOCY,
					 cos(pi*(h->srotor+180)/180)*SRSIZE +h->x+SRLOCX,sin(pi*(h->srotor+180)/180)*SRSIZE+h->y+SRLOCY, 255);
			// teken gewone rotor
			line(page[pn], h->x+RLOCX+cos(pi*h->rotor/180)*RSIZE, h->y+RLOCY, h->x+RLOCX-cos(pi*h->rotor/180)*RSIZE,h->y+RLOCY, 255);
		}
		if(h->type == OLDPLANE) 
			// teken propellor
			line(page[pn], h->x+PLOCX, h->y+PLOCY+sin(pi*h->rotor/180)*PSIZE, h->x+PLOCX, h->y+PLOCY-sin(pi*h->rotor/180)*PSIZE, 255);
	}
	else
	{
		if(h->type==0) {
			line(page[pn], cos(pi*h->srotor/180)*SRSIZE+h->x+h->bmp->w-SRLOCX,sin(pi*h->srotor/180)*SRSIZE+h->y+SRLOCY,
					 cos(pi*(h->srotor+180)/180)*SRSIZE+h->x+h->bmp->w-SRLOCX,sin(pi*(h->srotor+180)/180)*SRSIZE+h->y+SRLOCY,255);
			// teken gewone rotor
			line(page[pn], h->x+h->bmp->w-RLOCX-cos(pi*h->rotor/180)*RSIZE, h->y+RLOCY, h->x-RLOCX+h->bmp->w+cos(pi*h->rotor/180)*RSIZE,h->y+RLOCY, 255);
		}
		if(h->type == OLDPLANE) 
			// teken propellor
			line(page[pn], h->x+h->bmp->w-PLOCX, h->y+PLOCY+sin(pi*h->rotor/180)*PSIZE, h->x+h->bmp->w-PLOCX, h->y+PLOCY-sin(pi*h->rotor/180)*PSIZE, 255);
		draw_sprite_h_flip(page[pn],h->bmp, h->x,h->y);
	}

#define MAX_LIGHTS 20
#define HDOORX 70
#define HDOORY 5
#define RIGHT_SIDE 0
#define LEFT_SIDE 1
#define PLOCX 77        // Propellor
#define PLOCY 19
#define PSIZE 8
#define SLOCX 1         // Rook
#define SLOCY 19
*/

#define SRLOCX 8        // Staartrotor
#define SRLOCY 13
#define SRSIZE 8
#define RLOCX 70        // Hoofdrotor
#define RLOCY 2
#define RSIZE 60

void Player::draw(int xofs, int yofs)
{
	BITMAP* b = (BITMAP*) data[BMP_HELICOPTER].dat;
	int c= makecol(255, 255, 255);

	draw_sprite(double_buffer, b, int(x) - xofs - b->w/2,
		double_buffer->h - int(y) + yofs - b->h/2);

	xofs += b->w/2;
	yofs -= b->h/2;

	// sine wave !! (copied from paraclone :)
	// teken staartrotor
	line(double_buffer, int(cos(PI*srotor/128)*SRSIZE)+int(x)+SRLOCX-xofs,double_buffer->h-int(y)-int(sin(PI*srotor/128)*SRSIZE)+SRLOCY+yofs,
			 int(-cos(PI*srotor/128)*SRSIZE)+(int)x+SRLOCX-xofs,double_buffer->h-int(y)+int(sin(PI*srotor/128)*SRSIZE)+SRLOCY+yofs, c);

	// teken gewone rotor
	line(double_buffer, int(x)+int(cos(PI*rotor/128)*RSIZE)+RLOCX-xofs, double_buffer->h-int(y)+RLOCY+yofs, int(x)-int(cos(PI*rotor/128)*RSIZE)+RLOCX-xofs,double_buffer->h-int(y)+RLOCY+yofs, c);

	if (troll_unlocked)
	{
		textout(double_buffer, font, "TROLL UNLOCKED", int(x) - xofs - b->w/2, double_buffer->h - int(y) + yofs - b->h/2, makecol(255, 255, 255));
	}

	static const char* msg[] = 
	{
		"SuperKill",
		"MonsterKill",
		"MegaKill",
		"GigaKill",
		"TeraKill",
		"KILLING SPREE",
		"MONSTER KILLING SPREE",
		"SUPER POWER KILL",
		"mega SUPER power KILL",
		"double mega SUPER power KILL",
	};

	if (num_monsterkill>0)
	{
		int m = MIN(num_monsterkill, SIZEOF(msg))-1;
		BITMAP* bmp = create_bitmap(text_length(font, msg[m]), text_height(font));
		if (bmp)
		{
			text_mode(bitmap_mask_color(bmp));
			textout(bmp, font, msg[m], 0, 0, makecol(255, 255, 255));

			int w = int(zoom*bmp->w);
			int h = int(zoom*bmp->h);
			int x = (double_buffer->w - w)/2;
			int y = (double_buffer->h - h)/2;

			stretch_sprite(double_buffer, bmp, x, y, w, h);

			destroy_bitmap(bmp);
		}
	}
}
