/* Author: Tobi Vollebregt */

/*  Trolls'n Towers: Tobi Vollebregt's TINS Is Not SpeedHack 2005 entry.
 *  Copyright (C) 2005  Tobi Vollebregt
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See `License.txt', which contains a verbatim copy of the
 *  GNU General Public License, for details.
 *
 *  Please send your reaction to: tobivollebregt@hotmail.com
 */

// undefine this to remove (possibly slow) dithering code
// the value is the width of the lookup table bitmap
#define DITHER_15_16    32

#include <allegro.h>
#include "skybox.h"

#define H   (3*480)

#define R   (0<<16)
#define G   (64<<16)
#define B   (128<<16)
#define DR  (-R/H)
#define DG  (-G/H)
#define DB  (-B/H)

inline void Skybox::draw_no_dither(int, int y)
{
	int r = R+DR*y, g = G+DG*y, b = B+DB*y;

	double_buffer->clip = FALSE;

	for (y = double_buffer->h-1; y >= 0; --y)
	{
		hline(double_buffer, 0, y, double_buffer->w-1, makecol(r>>16, g>>16, b>>16));
		if ((r += DR) < 0) r = 0;
		if ((g += DG) < 0) g = 0;
		if ((b += DB) < 0) b = 0;
	}

	double_buffer->clip = TRUE;
}

#ifdef DITHER_15_16

inline void Skybox::draw_dither(int x, int y) // only 15/16 bit color
{
	int r = R+DR*y, g = G+DG*y, b = B+DB*y;
	BITMAP* bmp = create_bitmap(DITHER_15_16, 1);

	if (!bmp)
	{
		draw_no_dither(x, y);
		return;
	}

	bmp->clip = FALSE;
	double_buffer->clip = FALSE;

	for (y = double_buffer->h-1; y >= 0; --y)
	{
		for (x = 0; x < DITHER_15_16; ++x)
			putpixel(bmp, x, 0, (bitmap_color_depth(double_buffer) == 15 ?
				makecol15_dither : makecol16_dither)(r>>16, g>>16, b>>16, x, y));

		for (x = 0; x < double_buffer->w; x += DITHER_15_16)
			blit(bmp, double_buffer, 0, 0, x, y, DITHER_15_16, 1);

		if ((r += DR) < 0) r = 0;
		if ((g += DG) < 0) g = 0;
		if ((b += DB) < 0) b = 0;
	}

	double_buffer->clip = TRUE;
	destroy_bitmap(bmp);
}

#endif // defined (DITHER_15_16)

void Skybox::draw(int x, int y)
{
 #ifdef DITHER_15_16
	if (bitmap_color_depth(double_buffer)/8 == 2) draw_dither(x, y);
	else
 #endif // defined (DITHER_15_16)
		draw_no_dither(x, y);

	// and overlay some stars if we're high enough TODO
	/*if (y > H/2 && y < H + double_buffer->h)
	{
		drawing_mode(DRAW_MODE_TRANS);

		for (int i = 0; i < SIZEOF(stars); ++i)
		{
			if (c > 0)
			{
				set_trans_blender(0, 0, 0, c);
			}
		}
			int alpha = 255 * (y - H/2) / (H/2);


		solid_mode();
	}

	if (y >= H + double_buffer->h)
	{
	}*/
}
