/* Author: Tobi Vollebregt */

/*  Trolls'n Towers: Tobi Vollebregt's TINS Is Not SpeedHack 2005 entry.
 *  Copyright (C) 2005  Tobi Vollebregt
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See `License.txt', which contains a verbatim copy of the
 *  GNU General Public License, for details.
 *
 *  Please send your reaction to: tobivollebregt@hotmail.com
 */

#include <allegro.h>
#include "data.h"
#include "troll.h"
#include "particle.h"

// for troll (x,y) is at center bottom of sprite

#define GRAVITY (-0.2)

Troll::Troll(float X, float Y): x(X), y(Y)
{
	xs=ys=0.0;
}

bool Troll::update()
{
	GameObject* previous_hit = prev_hit;
	bool hit = false;
	BITMAP* b = (BITMAP*) data[BMP_TROLL].dat;
	GameObject* obj;
	for (int i = 0; i < GetNumGameObjects(); ++i)
		if ((obj = GetGameObject(i)) && obj != this && obj->hit(int(x), int(y)))
		{ prev_hit = obj;  hit = true;}

	x += xs;
	y += ys;
	ys += GRAVITY;
	if (y < 0 || hit)
	{
		xs = 0.1 * xs;
		ys = -0.5 * ys;
		if (y < 0) y = 0;
		if (previous_hit != prev_hit)
			for (int j = 0; j < 10; ++j)
			{
				int c = rand() & 63;
				AddGameObject2(new Particle(int(x)-b->w/2+rand()%b->w,int(y),
					RANDOM(-2.0,2.0),RANDOM(-2.0,2.0),50,10,makecol(c, c, c)));
			}
	}
	return true; // trolls are indestructable
}

void Troll::draw(int xofs, int yofs)
{
	BITMAP* b = (BITMAP*) data[BMP_TROLL].dat;

	draw_sprite(double_buffer, b, int(x)-xofs-b->w/2, double_buffer->h-int(y+0.5)+yofs-b->h);
}

bool Troll::hit(int hx, int hy)
{
	BITMAP* b = (BITMAP*) data[BMP_TROLL].dat;

	int bx = hx - int(x) + b->w/2;
	int by = b->h + int(y) - hy;
	int c = getpixel(b, bx, by);

	if (c >= 0 && c != bitmap_mask_color(b))
	//if (hx > int(x) - b->w/2 && hx < int(x) + b->w/2 &&
	//    hy > int(y) - b->h/2 && hy < int(y) + b->h/2)
	{
		/*for (int i = 0; i < 10; ++i)
		{
			AddGameObject2(new Particle(hx,hy,RANDOM(-2.0,2.0),RANDOM(-2.0,2.0),50,10,
				explosion_color[rand()&255]));
		}*/
		return true;
	}
	return false;
}
